/*
 * pfcx.c -- Detects specified planet or ship FCs and changes them to random numbers
 *
 * Copyright (C) 2000 Heiko Schlenker <hschlen@gmx.de>
 *
 * Author: Heiko Schlenker <hschlen@gmx.de>
 * Version: $Revision: 1.5 $
 * Status: quick & dirty
 *
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <phostpdk.h>

#define MAXINBUFSIZE 8

static const char rcsid[] = "$Id: pfcx.c,v 1.5 2000-07-31 18:38:46+02 hschlen Exp hschlen $";
const char PFCX_VERSION[] = "$Revision: 1.5 $";
const char ReleaseDate[] = "$Date: 2000-07-31 18:38:46+02 $";
const char Notice[] = "Copyright (C) 2000 Heiko Schlenker.\n\
Pfcx comes with ABSOLUTELY NO WARRANTY.\n\
This is free software. See COPYING for details.";


int
main(int argc, char *argv[])
{
    Uns16 i = 0, j = 0;
    Uns16 line = 0;
    Int16 dummy1 = 0;
    FILE *InFile = NULL;
    char FC[4] = {'\0'};
    char inbuf[MAXINBUFSIZE] = {'\0'};
    int returncode = 0;
    Boolean changedplanetfc = False;
    Boolean changedshipfc = False;


    /* Parse our command-line parameters. First parameter is the game directory. */
    switch (argc) {
    case 3:
	gRootDirectory = argv[2];
	/* FALL THROUGH */

    case 2:
	gGameDirectory = argv[1];
	break;
	/* FALL THROUGH */

    case 1:
	/* break; */

    default:
	printf("pfcx - Detects specified planet or ship FCs and changes them to random numbers.\n");
	printf("%s (%s)\n", PFCX_VERSION, ReleaseDate);
	printf("%s\n\n", Notice);
	printf("Usage: pfcx [GameDirectory [RootDirectory]]\n\n");
	exit(EXIT_FAILURE);
    }

    /* Initialize the PDK */
    InitPHOSTLib();

    if (!ReadGlobalData()) {
	FreePHOSTLib();
	exit(EXIT_FAILURE);
    }
    if (!ReadHostData()) {
	FreePHOSTLib();
	exit(EXIT_FAILURE);
    }

    gLogFile = OpenOutputFile("pfcx.log", TEXT_MODE | GAME_DIR_ONLY | NO_MISSING_ERROR);
    if (!gLogFile) {
	fprintf(stderr, "Error: Cannot open \"pfcx.log\"!\n");
	returncode = EXIT_FAILURE;
	goto error;
    }

    fprintf(gLogFile, "%s\n\n", rcsid);

    InFile = OpenInputFile("pfcx.ini", GAME_OR_ROOT_DIR | TEXT_MODE | NO_MISSING_ERROR);
    if (!InFile) {
	fprintf(stderr, "Error: Cannot open input file \"%s\"\n", "pfcx.ini");
	fprintf(gLogFile, "Error: Cannot open input file \"%s\"\n", "pfcx.ini");
	returncode = EXIT_FAILURE;
	goto error;
    }

    i = time(NULL);
    SetRandomSeed(i | 1);  /* set random seed */

    /* now scan through planets and ships and see which has a certain FC code */

    while (!feof(InFile)) {
	line++;
	if (0 == fgets(inbuf, 4, InFile)) {
	    if (!feof(InFile)) {
		fprintf(stderr,"Error: Error reading input file \"%s\" at line %u!\n", "pfcx.ini", line);
		fprintf(gLogFile, "Error: Error reading input file \"%s\" at line %u!\n", "pfcx.ini", line);
		returncode = EXIT_FAILURE;
		goto error;
	    }
	    break;
	}
	/* remove newline or carriage return */
	for (i = 0; i < MAXINBUFSIZE; i++) {
	    if (('\n' == inbuf[i]) || ('\r' == inbuf[i])) {
		inbuf[i] = '\0';
	    }
	}
	if (inbuf[0] != '\0') {
	    for (i = 1; i <= PLANET_NR; i++) {
		if (!IsPlanetExist(i)) {
		    continue;
		}
		PlanetFCode(i, FC);
		if ((strncmp(inbuf, FC, strlen(inbuf)) == 0)) {
		    /* scans first chars of the FC and compares them */
		    j = RandomRange(999);  /* return 3 digit number */
		    if (j < 100) {
			j += 100;  /* so we don't have to worry about zeros */
		    }
		    /* stuffs j into a string - there is no good C function for this */
		    sprintf(FC, "%u", j);
		    PutPlanetFC(i, FC);
		    changedplanetfc = True;
		    Info("Planet number %u has a \"%s\" setting!  Changed!", i, inbuf);
		}
	    }
	    for (i = 1; i <= SHIP_NR; i++) {
		if (!IsShipExist(i)) {
		    continue;
		}
		ShipFC(i, FC);
		if ((strncmp(inbuf, FC, strlen(inbuf)) == 0)) {
		    /* scans first chars of the FC and compares them */
		    j = RandomRange(999);  /* return 3 digit number */
		    if (j < 100) {
			j += 100;  /* so we don't have to worry about zeros */
		    }
		    /* stuffs j into a string - there is no good C function for this */
		    sprintf(FC, "%u", j);
		    PutShipFC(i, FC);
		    changedshipfc = True;
		    Info("Ship number %u has a \"%s\" setting!  Changed!", i, inbuf);
		}
	    }
	}
    }
    Info("");

    /* Close down. */
    if (True == changedplanetfc) {
	if (!Write_Planets_File(dummy1)) {
	    fprintf(stderr, "Error: Cannot write planet data file!\n");
	    fprintf(gLogFile, "Error: Cannot write planet data file!\n");
	    returncode = EXIT_FAILURE;
	}
    }

    if (True == changedshipfc) {
	if (!Write_Ships_File(dummy1)) {
	    fprintf(stderr, "Error: Cannot write ship data file!\n");
	    fprintf(gLogFile, "Error: Cannot write ship data file!\n");
	    returncode = EXIT_FAILURE;
	}
    }

error:

    if (InFile) {
	fclose(InFile);
    }
    if (gLogFile) {
	fclose(gLogFile);
    }
    FreePHOSTLib();

    exit(returncode);
}
